-- Freeciv - Copyright (C) 2007 - The Freeciv Project
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2, or (at your option)
--   any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.

-- This file is for lua-functionality that is specific to a given
-- ruleset. When freeciv loads a ruleset, it also loads script
-- file called 'default.lua'. The one loaded if your ruleset
-- does not provide an override is default/default.lua.


-- Place Ruins at the location of the destroyed city.
function city_destroyed_callback(city, loser, destroyer)
  city.tile:create_extra("Ruins", NIL)
  -- continue processing
  return false
end

signal.connect("city_destroyed", "city_destroyed_callback")

-- Add random labels to the map.
function place_map_labels()
  local mountains = 0
  local deep_oceans = 0
  local deserts = 0
  local glaciers = 0
  local selected_mountain = 0
  local selected_ocean = 0
  local selected_desert = 0
  local selected_glacier = 0

  -- Count the tiles that has a terrain type that may get a label.
  for place in whole_map_iterate() do
    local terr = place.terrain
    local tname = terr:rule_name()
    if tname == "Mountains" then
      mountains = mountains + 1
    elseif tname == "Deep Ocean" then
      deep_oceans = deep_oceans + 1
    elseif tname == "Desert" then
      deserts = deserts + 1
    elseif tname == "Glacier" then
      glaciers = glaciers + 1
    end
  end

  -- Decide if a label should be included and, in case it should, where.
  if random(1, 100) <= 75 then
    selected_mountain = random(1, mountains)
  end
  if random(1, 100) <= 75 then
    selected_ocean = random(1, deep_oceans)
  end
  if random(1, 100) <= 75 then
    selected_desert = random(1, deserts)
  end
  if random(1, 100) <= 75 then
    selected_glacier = random(1, glaciers)
  end

  -- Place the included labels at the location determined above.
  for place in whole_map_iterate() do
    local terr = place.terrain
    local tname = terr:rule_name()

    if tname == "Mountains" then
      selected_mountain = selected_mountain - 1
      if selected_mountain == 0 then
        place:set_label(_("Highest Peak"))
      end
    elseif tname == "Deep Ocean" then
      selected_ocean = selected_ocean - 1
      if selected_ocean == 0 then
        place:set_label(_("Deep Trench"))
      end
    elseif tname == "Desert" then
      selected_desert = selected_desert - 1
      if selected_desert == 0 then
        place:set_label(_("Scorched Spot"))
      end
    elseif tname == "Glacier" then
      selected_glacier = selected_glacier - 1
      if selected_glacier == 0 then
        place:set_label(_("Frozen Lake"))
      end
    end
  end

  return false
end

signal.connect("map_generated", "place_map_labels")


-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function action_finished_unit_callback(action, result, actor, target)
  if actor then
    local nmAction = action:rule_name()
    if nmAction == 'User Action 1' then
      run_convert_unit(actor, target)
    elseif nmAction == 'User Action 2' then
      run_watchtower(actor, target)
    end
  end
  return false
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function run_watchtower(actor, target)
  local has_begin = actor.tile:has_extra('Watchtower_BEGIN')
  local has_ready = actor.tile:has_extra('Watchtower_READY')
  if not has_begin and not has_ready then
    actor.tile:create_extra('Watchtower_BEGIN')
    notify_watchtower_begin(actor)
  end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function has_unit_type_name(unit, utype_name)
  return (unit.utype.id == find.unit_type(utype_name).id)
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function run_convert_unit(actor, target)
  local unit_new = nil
  local moves_left = 0
  if has_unit_type_name(actor, 'Explorer') then
    unit_new = find.unit_type('Raft')
  elseif has_unit_type_name(actor, 'Raft') then
    unit_new = find.unit_type('Explorer')
  end
  if unit_new and unit_new:can_exist_at_tile(target) then
    actor.owner:create_unit(target, unit_new, 0, actor:get_homecity(), moves_left)
    actor:kill('disbanded', actor.owner)
  end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function player_phase_begin_callback(player)
  services_watchtowers(player)
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function services_watchtowers(player)
  for unit in player:units_iterate() do
    local unit_type = unit.utype:rule_name()
    if unit_type == 'Explorer' then
      if unit.tile:has_extra('Watchtower_BEGIN') then
        unit.tile:remove_extra('Watchtower_BEGIN')
        unit.tile:create_extra('Watchtower_READY')
        unit:movement_disallow()
        notify_watchtower_ready(unit)
      elseif unit.tile:has_extra('Watchtower_READY') then
        unit.tile:remove_extra('Watchtower_READY')
        unit:movement_allow()
        notify_watchtower_done(unit)
      end
    end
  end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function unit_loss_watchtowers(unit, player, reason)
  local unit_type = unit.utype:rule_name()
  if unit_type == 'Explorer' then
    unit.tile:remove_extra('Watchtower_BEGIN')
    unit.tile:remove_extra('Watchtower_READY')  
  end
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function notify_watchtower_begin(actor)
  notify.event(actor.owner, actor.tile, E.UNIT_BECAME_VET, 
    string.format('%s will build a watchtower at coordinates %s.',
      actor.utype:name_translation(), 
      actor.tile:link_text()))
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function notify_watchtower_ready(actor)
  notify.event(actor.owner, actor.tile, E.UNIT_BECAME_VET, 
    string.format('%s is observing the %s area. 1 turn without movement.',
      actor.utype:name_translation(), 
      actor.tile:link_text()))
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function notify_watchtower_done(actor)
  notify.event(actor.owner, actor.tile, E.UNIT_BECAME_VET, 
    string.format('%s has finished mapping the %s area. Movement is allowed.',
      actor.utype:name_translation(), 
      actor.tile:link_text()))
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
function notify_conversion(actor, unit_new, target)
  notify.event(actor.owner, target.tile, E.UNIT_BUILT, 
    string.format('Unit %s was converted to %s in %s.',
      actor.utype:name_translation(), 
      unit_new:name_translation(), 
      target:link_text()) )
end
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=
signal.connect('unit_lost', 'unit_loss_watchtowers')
signal.connect('player_phase_begin', 'player_phase_begin_callback')
signal.connect('action_finished_unit_extras', 'action_finished_unit_callback')
signal.connect('action_finished_unit_self',   'action_finished_unit_callback')
signal.connect('action_finished_unit_tile',   'action_finished_unit_callback')
signal.connect('action_finished_unit_unit',   'action_finished_unit_callback')
signal.connect('action_finished_unit_units',  'action_finished_unit_callback')
signal.connect('action_finished_unit_city',   'action_finished_unit_callback')
-- ~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=~=

